// -*- C++ -*-
#include "Rivet/Analysis.hh"
#include "Rivet/Projections/ChargedFinalState.hh"


namespace Rivet {


  /// @brief Inelastic xsection in pp collisions at 13 TeV for charged particles in LHCb acceptance
  class LHCB_2018_I1665223 : public Analysis {
  public:

    /// Constructor
    //DEFAULT_RIVET_ANALYSIS_CTOR(LHCB_2018_I1665223);
    /// Constructor
    LHCB_2018_I1665223()
      : Analysis("LHCB_2018_I1665223"),
        _etaMin(2.0),
        _etaMax(5.0),
        _pMin(2.0)
    {    }


    /// @name Analysis methods
    //@{

    /// Book histograms and initialise projections before the run
    void init() {

      // Initialise and register projections
      declare(ChargedFinalState(Cuts::etaIn(_etaMin,_etaMax)), "lbCFS");

      // Book histograms
      _h_ppInel = bookHisto1D(1, 1, 1);

    }


    /// Perform the per-event analysis
    void analyze(const Event& event) {

      const double             weight  = event.weight();
      const ChargedFinalState   &cfs    = applyProjection<ChargedFinalState> (event, "lbCFS");

      // eliminate non-inelastic events and empty events in LHCb
      if (cfs.particles().size() == 0) vetoEvent;
      foreach (const Particle &myp, cfs.particles()){
          if (hasLongLivedParent(myp)) continue;
          if (!isLongLivedParticle(myp)) continue;
          // only particles with p>2 GeV measured
          if (myp.momentum().p()/GeV < _pMin) continue;
          // histo gets filled only for inelastic events (at least one prompt charged particle)
          _h_ppInel->fill(sqrtS(), weight);
          break;
      } //end loop on particles
    }


    /// Normalise histograms etc., after the run
    void finalize() {

      scale(_h_ppInel, crossSection()/millibarn/sumOfWeights()); // norm to cross section (one-sided LHCb)

    }

    //@}

  private:

  	// list of PDG IDs for particle with lifetimes higher than 0.03 ns (3.E-11 s) - long lived particles according to LHC MB&UE WG
    static const int longLivedPids[20];

  	bool isLongLivedParticle(const Particle& p) {
  		// stable long-lived final-state charged particles in LHCb
  		const int stablePids[9] = {11,13,211,321,2212,3112,3222,3312,3334};
  		for (int i = 0; i<9; i++) {
  			if (p.abspid() == stablePids[i]) return true;
  		}
  		return false;
  	}

    bool hasLongLivedParent(const Particle& p)
    {
    	bool ret = false;
    	for (int i = 0; i<20; i++) {
    		if (p.hasParentWith(Cuts::abspid == longLivedPids[i])) { ret = true; break; };
    	}
    	return ret;
    }

    /// @name Histograms
    //@{
    Histo1DPtr _h_ppInel;
    double _etaMin, _etaMax, _pMin;

    //@}


  };

  const int LHCB_2018_I1665223::longLivedPids[20] = {3334,3322,3312,3222,3122,3112,2212,2112,321,310,130,211,20022,480000000,11,12,13,14,16,22};

  // The hook for the plugin system
  DECLARE_RIVET_PLUGIN(LHCB_2018_I1665223);


}
